#include <ros/ros.h>
#include <iostream>
#include <numeric>
#include <stdlib.h>
#include <fstream>
#include <sstream>
#include <string>
#include <cv_bridge/cv_bridge.h>
#include <opencv2/opencv.hpp>
#include <boost/filesystem.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>
#include "libhaloc/lc.h"
#include <sensor_msgs/image_encodings.h>
#include <image_transport/image_transport.h>
 
using namespace std;
namespace fs=boost::filesystem;
 
//Store all constants for image encodings in the enc namespace to be used later.
namespace enc = sensor_msgs::image_encodings;
//Declare a string with the name of the window that we will create using OpenCV where processed images will be displayed.
static const char WINDOW[] = "Image Processed";
 
//Use method of ImageTransport to create image publisher
image_transport::Publisher pub;
  
int  c = 0;
char buffer[1000];
char frame_name[20];
int count_frames = 0;
/**
 * Executes libhaloc over a set of images
 */
class Mono
{
  public:
 
    /** \brief Mono class constructor
     */
    Mono(ros::NodeHandle nh, ros::NodeHandle nhp) : nh_(nh), nh_private_(nhp)
    {
      // Read the parameters
      readParams();
    }
 
    /** \brief Processes the image dataset
     */
    void processData()
    {
  
      // Init Haloc
      haloc::LoopClosure::Params lc_params;
      lc_params.work_dir = output_path_;
      lc_params.desc_type = desc_type_;
      lc_params.desc_matching_type = desc_matching_type_;
      lc_params.desc_thresh_ratio = desc_thresh_ratio_;
      lc_params.epipolar_thresh = epipolar_thresh_;
      lc_params.min_neighbor = min_neighbor_;
      lc_params.n_candidates = n_candidates_;
      lc_params.min_matches = min_matches_;
      lc_params.min_inliers = min_inliers_;
      lc_.setParams(lc_params);
      lc_.init();
 
       
    }
 
    void imageCallback(const sensor_msgs::ImageConstPtr& original_image)
    {
      cv_bridge::CvImagePtr cv_ptr;
      try
      {
        //Always copy, returning a mutable CvImage
        //OpenCV expects color images to use BGR channel order.
        cv_ptr = cv_bridge::toCvCopy(original_image, enc::BGR8);
      }
      catch (cv_bridge::Exception& e)
      {
        //If there is an error during conversion, display it
        ROS_ERROR("tutorialROSOpenCV::main.cpp::cv_bridge exception: %s", e.what());
        return;
      }     
      resize(cv_ptr->image, cv_ptr->image, Size(), 0.2, 0.2, CV_INTER_AREA);
      cv::imshow(WINDOW, cv_ptr->image);
      waitKey(3);
      cv::Mat image = cv_ptr->image;
      pub.publish(cv_ptr->toImageMsg());
 
      //Save every n-th frame
      count_frames++;
 
      if(count_frames > 10)
      {
        count_frames=0;
        
        
        //Save images
        vector<int> compression_params;
        compression_params.push_back(CV_IMWRITE_PNG_COMPRESSION);
        compression_params.push_back(9);
        sprintf(buffer,"/home/student/ros/src/libhaloc/slike/frame%06u.bmp",c);
        imwrite(buffer, image, compression_params);
        cv::waitKey(3);
        
 
        // Iterate over all images
        //int found_lc = 0;
        //vector<string> image_filenames;
        cv::imshow("lc", cv_ptr->image);
        waitKey(3);
        // Get image
        sprintf(frame_name, "frame%06u.bmp",c);
        string cur_filename = frame_name;
        //Mat img = imread(img_dir_+"/"+cur_filename, CV_LOAD_IMAGE_COLOR);
        //resize(img, img, Size(), 0.2, 0.2, CV_INTER_AREA);
        //image_filenames.push_back(cur_filename);
  
        // Set the new image
        lc_.setNode(image);
       
        // Get the loop closure (if any)
        int img_lc = -1;
        bool valid = lc_.getLoopClosure(img_lc);
 
        /*     
        string lc_filename = "";
        if (img_lc >= 0)
        {
        lc_filename = image_filenames[img_lc];
        }
        */
 
        // Log
        ROS_INFO_STREAM( cur_filename << " cl with " << img_lc << ": " << valid);
         
        /*//Print number of lcs
        if(!(ros::ok()))
        {
        // Finalize the loop closure class
        lc_.finalize();
 
        // Print the results
        ROS_INFO_STREAM("FOUND #LC: " << found_lc);
       
        }
        */
        //Iterate frame
        c++;
      }
    }
 
  protected:
 
    // Node handlers
    ros::NodeHandle nh_;
    ros::NodeHandle nh_private_;
 
  private:
 
    // Properties
    string tmp_id_, img_dir_, desc_type_, desc_matching_type_, output_path_, gt_file_;
    double desc_thresh_ratio_, epipolar_thresh_;
    int min_neighbor_, n_candidates_, min_matches_, min_inliers_, gt_tolerance_;
    haloc::LoopClosure lc_;
 
    /** \brief Read the parameters from the ros parameter server
     */
    void readParams()
    {
      nh_private_.param("tmp_id", tmp_id_, std::string(""));
      nh_private_.param("output_path", output_path_, std::string(""));
      nh_private_.param("img_dir", img_dir_, std::string(""));
      nh_private_.param("gt_file", gt_file_, std::string(""));
      nh_private_.param("desc_type", desc_type_, std::string("SIFT"));
      nh_private_.param("desc_matching_type", desc_matching_type_, std::string("CROSSCHECK"));
      nh_private_.getParam("desc_thresh_ratio", desc_thresh_ratio_);
      nh_private_.getParam("min_neighbor", min_neighbor_);
      nh_private_.getParam("n_candidates", n_candidates_);
      nh_private_.getParam("min_matches", min_matches_);
      nh_private_.getParam("min_inliers", min_inliers_);
      nh_private_.getParam("epipolar_thresh", epipolar_thresh_);
      nh_private_.getParam("gt_tolerance", gt_tolerance_);
 
      // Log
      cout << "  tmp_id            = " << tmp_id_ << endl;
      cout << "  output_path       = " << output_path_ << endl;
      cout << "  img_dir           = " << img_dir_ << endl;
      cout << "  desc_type         = " << desc_type_ << endl;
      cout << "  desc_thresh_ratio = " << desc_thresh_ratio_ << endl;
      cout << "  min_neighbor      = " << min_neighbor_ << endl;
      cout << "  n_candidates      = " << n_candidates_ << endl;
      cout << "  min_matches       = " << min_matches_ << endl;
      cout << "  min_inliers       = " << min_inliers_ << endl;
      cout << "  epipolar_thresh   = " << epipolar_thresh_ << endl;
      cout << "  gt_tolerance      = " << gt_tolerance_ << endl;
 
      // Files path sanity check
      if (output_path_[output_path_.length()-1] != '/')
        output_path_ += "/";
 
      // Sanity checks
      if (!fs::exists(img_dir_) || !fs::is_directory(img_dir_))
      {
        ROS_ERROR_STREAM("[HashMatching:] The image directory does not exists: " <<
                         img_dir_);
      }
    }
};
 
// Main entry point
int main(int argc, char **argv)
{
  ros::init(argc,argv,"example_mono");
  ros::NodeHandle nh;
  ros::NodeHandle nh_private("~");
 
   
  // Init node
  Mono mono(nh,nh_private);
 
  // Process the data
  mono.processData();
 
   //Stream
  image_transport::ImageTransport it(nh); 
  cv::namedWindow(WINDOW, CV_WINDOW_AUTOSIZE);
  image_transport::Subscriber sub = it.subscribe("/camera/left/image_rect", 1, &Mono::imageCallback, &mono);
  cv::destroyWindow(WINDOW);
  pub = it.advertise("camera/image_processed", 1);

  // Subscription is handled at start and stop service callbacks.
  ros::spin();
 
  return 0;
}